declare module 'requirejs-esmodule';

type IDeepRequired<T> = {
  [P in keyof T]-?: T[P] extends IPrimitive ? T[P] : IDeepRequired<T[P]>;
};

interface IFeatureOption {
  custom?: {
    [p: string]: {
      type: 'input',
      label: string
    } | {
      type: 'switch',
      label: string
    } | {
      type: 'radio',
      label: string,
      form: Array<{ name: string, value: string }>
    } | {
      type: 'select',
      label: string,
      form: Array<{ name: string, value: string }>
    }
  };
}

interface IState {
  // 名称
  name: string;

  // 样式
  style: Partial<React.CSSProperties & IBaseStyle>;

  // 属性
  property?: {
    color?: string[] | string;　// 颜色
    title?: { // 标题
      show: boolean;　// 显示
      text: string; // 标题文字
      style: {
        textAlign: 'left' | 'center' | 'right';　// 对齐
        fontSize: string; // 字体大小
        color: string; // 颜色
      };
    };
    text?: { // 文本
      fontSize: string;　// 字体大小
      fontStyle: 'normal' | 'italic'; // 字体样式
      fontWeight: 'normal' | 'lighter' | 'bold'; // 字体重量
      textAlign: 'left' | 'center' | 'right';　// 对齐
      lineHeight: string;　// 行高
      color: string; // 文字颜色
      fontFamily: 'none' | '...'; // 字体
    };
    xAxis?: IAxis<'X'>;　// x轴
    yAxis?: IAxis<'Y'>;　// y轴
    list?: IList; // 列表
    img?: string; // 图片(路径)
    video?: string; // 视频(路径)
    custom?: { // 自定义属性
      [p: string]: any;
    }
  };

  // 数据
  data?: {
    type: 'json' | 'excel' | 'api' | 'database';　// 当前数据的格式 (兼容dataType)
    json?: any;　// json格式的数据
    excel?: { // excel表格格式的数据
      columns: Array<{ key: string; name: string }>;　// 表头
      rows: object[];　// 行
    };
    api?: { // api的配置
      data: any,　// api返回的数据
      form: {
        url: string;　// api路径
        static: boolean;　// 是否保存静态数据不再发起请求
        fields?: string[];　// 字段
        xAxis?: string; // x轴
        yAxis?: Array<{ id: number | string; name: string; value: string[] }>;　// y轴序列
      }
    };　// api格式的数据
    database?: any;　// 数据库格式的数据
  };
}

interface IAxis<T extends 'X' | 'Y'> {
  show: boolean;　// 显示
  position: T extends 'X' ? 'top' | 'bottom' : 'left' | 'right';　// 坐标轴位置
  name: string; // 坐标轴名称
  axisLine: {
    show: boolean;
    lineStyle: {
      color: string;
      width: number;
    }
  };
  axisTick: {
    show: boolean;
    length: number;
    lineStyle: {
      color: string;
      width: number;
    }
  };
  axisLabel: {
    show: boolean;
    rotate: number;
    fontSize: number;
    fontFamily: string;
  };
  splitLine: {
    show: boolean;
    interval: number;
    lineStyle: {
      color: string[];
      width: number;
      type: 'solid' | 'dashed' | 'dotted';
    }
  };
}

interface IList {
  border: {
    borderWidth: string;
    borderColor: string;
    borderStyle: string;
  };
  // header 配置
  header: {
    // 是否显示列表头。显示时，props.data 数据的第一个数据集为列表头数据
    show: boolean;
    // 表头行样式
    style: {
      height: number;
      // 背景颜色
      backgroundColor: string;
      // 背景图片
      backgroundImage: string;
    },
    // 单元格style ，除了width外，其余参照css样式表
    cellStyle: {
      // 字体颜色
      color: string;
      border: string;
    }
  };
  // body 配置
  body: {
    row: {
      spacing: number;
      rowCheckBox: boolean;
      style: {
        height: number;
        backgroundColor: string;
        backgroundImage: string;
      };
      /**
       * 每一行的style交替出现配置项
       */
      visual: {
        // 开关
        show: boolean;
        /**
         * 交替间隔
         *    0 相当于show设置为false
         *    1 每1行交替一次
         *    2 每2行交替一次
         *    ...依次类推
         */
        interval: number;
        // 用于覆盖body.row.style相关属性的配置项
        style: {
          backgroundColor: string;
          backgroundImage: string;
        }
      };
      // 图形是否不响应和触发鼠标事件，默认为 false，即响应和触发鼠标事件。
      silent: {
        show: boolean;
        // 鼠标交互时的样式，目前只支持hover
        style: {
          backgroundColor: string;
          backgroundImage: string;
        }
      }
    };
    cell: {
      style: {
        fontSize: number;
        minWidth: number;
        color: string;
        border: string;
        /** 每一列的单元格的宽度
         * 可选
         *    'auto'：自动；
         *    10 / '10px'：10像素；
         *    [10, 20, 10]：每一列依次取数组的值，数组长度小于列数时多余的列默认'auto'
         */
        width: string | number | number[];
      };
      iconStyle: {
        width: number;
        height: string;
      }
    }
  };
  // 列表滚动速度
  speed: number;
  // 是否开启列表滚动
  isScroll: boolean;
}

interface IFeatureOption {
  box?: boolean;
  path?: string;
}
